/*
           LTR43.
            ,    
           :
    ltr43_sec_pulse  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd/LtrServer (     )
     :    ,       

          1,    0,     
     (  ,     
      )    .

           Windows   CTRL+C  Linux.

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr43api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .    ,   
           ( Consolas).
*/



#include <stdio.h>
#include <stdlib.h>
#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#else
#include <signal.h>
#include <unistd.h>
#endif

#include "ltr/include/ltr43api.h"

#define RECV_BLOCK_SIZE 10000


/*      */
static int f_out = 0;

#ifndef _WIN32
/*     Linux */
static void f_abort_handler(int sig) {
    f_out = 1;
}
#endif



typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;

/*    .   ,  
 *   :
 * 1  -   ( 1  16)
 * 2  -   
 * 3  - ip-  */
static int f_get_params(int argc, char** argv, t_open_param* par) {
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;


    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3) {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++) {
            if ((a[i]<0) || (a[i] > 255)) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err) {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}
/*------------------------------------------------------------------------------------------------*/
int main(int argc, char *argv[]) {
    INT err = LTR_OK;
    TLTR43 hltr43;
    t_open_param par;

#ifndef _WIN32
    struct sigaction sa;
    /*   Linux      ,
           */
    sa.sa_handler = f_abort_handler;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGABRT, &sa, NULL);
#endif
#ifdef _WIN32
    /*         Windows*/
    setlocale(LC_ALL, "");
#endif

    err = f_get_params(argc, argv, &par);

    if (err==LTR_OK) {
        LTR43_Init(&hltr43);
        err = LTR43_Open(&hltr43, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
        if (err != LTR_OK) {
            fprintf(stderr, "     .  %d (%s)\n",
                    err, LTR43_GetErrorString(err));
        } else {
            DWORD wr_val = 0x55555555;

            printf("  :\n");
            printf("  :        %s\n", hltr43.ModuleInfo.Name);
            printf("   :  %s\n", hltr43.ModuleInfo.Serial);
            printf("   : %s\n", hltr43.ModuleInfo.FirmwareVersion);
            printf("   :   %s\n", hltr43.ModuleInfo.FirmwareDate);


            hltr43.IO_Ports.Port1 = LTR43_PORT_DIR_OUT;
            hltr43.IO_Ports.Port2 = LTR43_PORT_DIR_OUT;
            hltr43.IO_Ports.Port3 = LTR43_PORT_DIR_OUT;
            hltr43.IO_Ports.Port4 = LTR43_PORT_DIR_OUT;

            err = LTR43_Config(&hltr43);
            if (err != LTR_OK) {
                fprintf(stderr, "  .  %d: %s\n",
                        err, LTR43_GetErrorString(err));
            }
            printf("  .    %s\n",
#ifdef _WIN32
                   " "
#else
                   "CTRL+C"
#endif
                   );

            while ((err == LTR_OK) && !f_out) {

                err = LTR43_WritePort(&hltr43, wr_val);
                if (err!=LTR_OK) {
                    fprintf(stderr, "    .  %d:%s\n",
                            err, LTR43_GetErrorString(err));
                } else {
                    LTRAPI_SLEEP_MS(500);
                    wr_val ^= 0xFFFFFFFF;
                }

#ifdef _WIN32
                /*      */
                if (err==LTR_OK) {
                    if (_kbhit())
                        f_out = 1;
                }
#endif
            }
        }

        if (LTR43_IsOpened(&hltr43) == LTR_OK)
            LTR43_Close(&hltr43);
    }
    return err;
}
